﻿/// <reference path="../typings/globals/lodash/index.d.ts" />

import * as mongoose from "mongoose";
import * as Rx from "rxjs";
var _ = require("lodash");

export default class Repository<TModel> {

  // Intersection Type: https://www.typescriptlang.org/docs/handbook/advanced-types.html
  constructor(private documentModel:mongoose.Model<TModel & mongoose.Document>) { }

  getAll(): Rx.Observable<TModel[]> {

    return Rx.Observable.create((obs: Rx.Observer<TModel[]>) => {
      this.documentModel
        .find({})
        .exec((err, documents) => {
          if (err)
            return obs.error(err);

          obs.next(documents);
          obs.complete();
        });
    });
  }

  create(model: TModel) {
    return Rx.Observable.create((obs: Rx.Observer<TModel>) => {

      if (!_.isObject(model))
        return obs.error(new TypeError("Todo is not a valid object."));

      var ctor: {new(doc?: TModel, fields?: Object, skipInit?: boolean): mongoose.Document;} = this.documentModel;
      
      var documentModelInstance = new ctor(model);

      documentModelInstance.save((err, saved: mongoose.Document) => {
        if (err)
            return obs.error(err);

          obs.next(saved.toObject() as TModel);
          obs.complete();
      });
    });
  }

  deleteById(id: string) {
    return Rx.Observable.create((obs: Rx.Observer<TModel>) => {
      if (!_.isString(id))
          return obs.error(new TypeError("Id is not a valid string."));
      
      this.documentModel
        .findByIdAndRemove(id)
        .exec((err, deleted) => {
            if (err)
              return obs.error(err);

            obs.next(deleted.toObject() as TModel);
            obs.complete();
        });
    });
  }

}