﻿/// <reference path="typings/index.d.ts" />

import * as express from "express";
import { Request, Response, NextFunction } from "express";
import DBConfig from "./config/db.config";
import TodoRoutes from "./api/todo/routes/todo-routes";

var logger = require("morgan");
var bodyParser = require("body-parser");
var cors = require("cors");

var app = express();

var webAppUrl = "http://localhost:50771";

var corsOptions = {
  origin: webAppUrl,
  optionsSuccessStatus: 200 // some legacy browsers (IE11, various SmartTVs) choke on 204 
};

app.use(cors(corsOptions));

app.use(logger("dev"));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));

DBConfig.init();

app.use("/api", TodoRoutes.buildRouter());


// catch 404 and forward to error handler
app.use((req:Request, res:Response, next:NextFunction) => {
    var err:any = new Error("Not Found");
    err.status = 404;
    next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get("env") === "development") {
    app.use((err, req: Request, res: Response, next:NextFunction) => {
	    res.status(err.status || 500);
	    res.render("error", {
		    message: err.message,
		    error: err
	    });
    });
}

// production error handler
// no stacktraces leaked to user
app.use((err, req: Request, res: Response, next:NextFunction) => {
    res.status(err.status || 500);
    res.render("error", {
        message: err.message,
        error: {}
    });
});

var server = app.listen(3301, () => {
	console.log("Express server listening on port " + server.address().port);
});
